package java.net;

import java.security.PrivilegedAction;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.Objects;
import java.util.Collections;
import java.util.HashSet;
import java.io.IOException;
import java.io.FileDescriptor;
import sun.net.ResourceManager;
import java.util.Set;
import sun.net.ExtendedSocketOptions;

abstract class AbstractPlainDatagramSocketImpl extends DatagramSocketImpl
{
    int timeout;
    boolean connected;
    private int trafficClass;
    protected InetAddress connectedAddress;
    private int connectedPort;
    private static final String os;
    private static final boolean connectDisabled;
    static final ExtendedSocketOptions extendedOptions;
    private static final Set<SocketOption<?>> datagramSocketOptions;
    private static final Set<SocketOption<?>> multicastSocketOptions;
    
    AbstractPlainDatagramSocketImpl() {
        this.timeout = 0;
        this.connected = false;
        this.trafficClass = 0;
        this.connectedAddress = null;
        this.connectedPort = -1;
    }
    
    @Override
    protected synchronized void create() throws SocketException {
        ResourceManager.beforeUdpCreate();
        this.fd = new FileDescriptor();
        try {
            this.datagramSocketCreate();
        }
        catch (SocketException ex) {
            ResourceManager.afterUdpClose();
            this.fd = null;
            throw ex;
        }
    }
    
    @Override
    protected synchronized void bind(int n, InetAddress inetAddress) throws SocketException {
        this.bind0(n, inetAddress);
    }
    
    protected abstract void bind0(int p0, InetAddress p1) throws SocketException;
    
    @Override
    protected abstract void send(DatagramPacket p0) throws IOException;
    
    @Override
    protected void connect(InetAddress connectedAddress, int connectedPort) throws SocketException {
        this.connect0(connectedAddress, connectedPort);
        this.connectedAddress = connectedAddress;
        this.connectedPort = connectedPort;
        this.connected = true;
    }
    
    @Override
    protected void disconnect() {
        this.disconnect0(this.connectedAddress.holder().getFamily());
        this.connected = false;
        this.connectedAddress = null;
        this.connectedPort = -1;
    }
    
    @Override
    protected abstract int peek(InetAddress p0) throws IOException;
    
    @Override
    protected abstract int peekData(DatagramPacket p0) throws IOException;
    
    @Override
    protected synchronized void receive(DatagramPacket datagramPacket) throws IOException {
        this.receive0(datagramPacket);
    }
    
    protected abstract void receive0(DatagramPacket p0) throws IOException;
    
    @Override
    protected abstract void setTimeToLive(int p0) throws IOException;
    
    @Override
    protected abstract int getTimeToLive() throws IOException;
    
    @Deprecated
    @Override
    protected abstract void setTTL(byte p0) throws IOException;
    
    @Deprecated
    @Override
    protected abstract byte getTTL() throws IOException;
    
    @Override
    protected void join(InetAddress inetAddress) throws IOException {
        this.join(inetAddress, null);
    }
    
    @Override
    protected void leave(InetAddress inetAddress) throws IOException {
        this.leave(inetAddress, null);
    }
    
    @Override
    protected void joinGroup(SocketAddress socketAddress, NetworkInterface networkInterface) throws IOException {
        if (socketAddress == null || !(socketAddress instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        this.join(((InetSocketAddress)socketAddress).getAddress(), networkInterface);
        return;
    }
    
    protected abstract void join(InetAddress p0, NetworkInterface p1) throws IOException;
    
    @Override
    protected void leaveGroup(SocketAddress socketAddress, NetworkInterface networkInterface) throws IOException {
        if (socketAddress == null || !(socketAddress instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        this.leave(((InetSocketAddress)socketAddress).getAddress(), networkInterface);
        return;
    }
    
    protected abstract void leave(InetAddress p0, NetworkInterface p1) throws IOException;
    
    @Override
    protected void close() {
        if (this.fd != null) {
            this.datagramSocketClose();
            ResourceManager.afterUdpClose();
            this.fd = null;
        }
    }
    
    protected boolean isClosed() {
        return this.fd == null;
    }
    
    @Override
    protected void finalize() {
        this.close();
    }
    
    @Override
    public void setOption(int i, Object o) throws SocketException {
        if (this.isClosed()) {
            throw new SocketException("Socket Closed");
        }
        switch (i) {
            case 4102: {
                if (o == null || !(o instanceof Integer)) {
                    throw new SocketException("bad argument for SO_TIMEOUT");
                }
                int intValue = (int)o;
                if (intValue < 0) {
                    throw new IllegalArgumentException("timeout < 0");
                }
                this.timeout = intValue;
                return;
            }
            case 3: {
                if (o == null || !(o instanceof Integer)) {
                    throw new SocketException("bad argument for IP_TOS");
                }
                this.trafficClass = (int)o;
                break;
            }
            case 4: {
                if (o == null || !(o instanceof Boolean)) {
                    throw new SocketException("bad argument for SO_REUSEADDR");
                }
                break;
            }
            case 32: {
                if (o == null || !(o instanceof Boolean)) {
                    throw new SocketException("bad argument for SO_BROADCAST");
                }
                break;
            }
            case 15: {
                throw new SocketException("Cannot re-bind Socket");
            }
            case 4097:
            case 4098: {
                if (o == null || !(o instanceof Integer) || (int)o < 0) {
                    throw new SocketException("bad argument for SO_SNDBUF or SO_RCVBUF");
                }
                break;
            }
            case 16: {
                if (o == null || !(o instanceof InetAddress)) {
                    throw new SocketException("bad argument for IP_MULTICAST_IF");
                }
                break;
            }
            case 31: {
                if (o == null || !(o instanceof NetworkInterface)) {
                    throw new SocketException("bad argument for IP_MULTICAST_IF2");
                }
                break;
            }
            case 18: {
                if (o == null || !(o instanceof Boolean)) {
                    throw new SocketException("bad argument for IP_MULTICAST_LOOP");
                }
                break;
            }
            default: {
                throw new SocketException("invalid option: " + i);
            }
        }
        this.socketSetOption(i, o);
        return;
    }
    
    @Override
    public Object getOption(int i) throws SocketException {
        if (this.isClosed()) {
            throw new SocketException("Socket Closed");
        }
        Integer n;
        switch (i) {
            case 4102: {
                n = new Integer(this.timeout);
                break;
            }
            case 3: {
                n = this.socketGetOption(i);
                if ((int)n == -1) {
                    n = new Integer(this.trafficClass);
                    break;
                }
                break;
            }
            case 4:
            case 15:
            case 16:
            case 18:
            case 31:
            case 32:
            case 4097:
            case 4098: {
                n = this.socketGetOption(i);
                break;
            }
            default: {
                throw new SocketException("invalid option: " + i);
            }
        }
        return n;
    }
    
    private static Set<SocketOption<?>> datagramSocketOptions() {
        HashSet<StandardSocketOptions> s = new HashSet();
        s.add(StandardSocketOptions.SO_SNDBUF);
        s.add(StandardSocketOptions.SO_RCVBUF);
        s.add(StandardSocketOptions.SO_REUSEADDR);
        s.add(StandardSocketOptions.IP_TOS);
        s.addAll(ExtendedSocketOptions.datagramSocketOptions());
        return Collections.unmodifiableSet(s);
    }
    
    private static Set<SocketOption<?>> multicastSocketOptions() {
        HashSet<StandardSocketOptions> s = new HashSet();
        s.add(StandardSocketOptions.SO_SNDBUF);
        s.add(StandardSocketOptions.SO_RCVBUF);
        s.add(StandardSocketOptions.SO_REUSEADDR);
        s.add(StandardSocketOptions.IP_TOS);
        s.add(StandardSocketOptions.IP_MULTICAST_IF);
        s.add(StandardSocketOptions.IP_MULTICAST_TTL);
        s.add(StandardSocketOptions.IP_MULTICAST_LOOP);
        s.addAll(ExtendedSocketOptions.datagramSocketOptions());
        return Collections.unmodifiableSet(s);
    }
    
    private Set<SocketOption<?>> supportedOptions() {
        if ((this.getDatagramSocket() instanceof MulticastSocket)) {
            return AbstractPlainDatagramSocketImpl.multicastSocketOptions;
        }
        return AbstractPlainDatagramSocketImpl.datagramSocketOptions;
    }
    
    protected <T> void setOption(SocketOption<T> obj, T obj2) throws IOException {
        Objects.requireNonNull(obj);
        if (!this.supportedOptions().contains(obj)) {
            throw new UnsupportedOperationException("'" + obj + "' not supported");
        }
        if (!obj.type().isInstance(obj2)) {
            throw new IllegalArgumentException("Invalid value '" + obj2 + "'");
        }
        if (this.isClosed()) {
            throw new SocketException("Socket closed");
        }
        if (obj == StandardSocketOptions.SO_SNDBUF) {
            if ((int)obj2 < 0) {
                throw new IllegalArgumentException("Invalid send buffer size:" + obj2);
            }
            this.setOption(4097, obj2);
        }
        else if (obj == StandardSocketOptions.SO_RCVBUF) {
            if ((int)obj2 < 0) {
                throw new IllegalArgumentException("Invalid recv buffer size:" + obj2);
            }
            this.setOption(4098, obj2);
        }
        else if (obj == StandardSocketOptions.SO_REUSEADDR) {
            this.setOption(4, obj2);
        }
        else if (obj == StandardSocketOptions.IP_TOS) {
            int intValue2 = (int)obj2;
            if (intValue2 < 0 || intValue2 > 255) {
                throw new IllegalArgumentException("Invalid IP_TOS value: " + obj2);
            }
            this.setOption(3, obj2);
        }
        else if (obj == StandardSocketOptions.IP_MULTICAST_IF) {
            this.setOption(31, obj2);
        }
        else if (obj == StandardSocketOptions.IP_MULTICAST_TTL) {
            int intValue = (int)obj2;
            if (intValue < 0 || intValue > 255) {
                throw new IllegalArgumentException("Invalid TTL/hop value: " + obj2);
            }
            this.setTimeToLive((int)obj2);
        }
        else if (obj == StandardSocketOptions.IP_MULTICAST_LOOP) {
            this.setOption(18, obj2);
        }
        else if (AbstractPlainDatagramSocketImpl.extendedOptions.isOptionSupported(obj)) {
            AbstractPlainDatagramSocketImpl.extendedOptions.setOption(this.fd, obj, obj2);
        }
        else {
            throw new AssertionError("unknown option :" + obj);
        }
        return;
    }
    
    protected <T> T getOption(SocketOption<T> obj) throws IOException {
        Objects.requireNonNull(obj);
        if (!this.supportedOptions().contains(obj)) {
            throw new UnsupportedOperationException("'" + obj + "' not supported");
        }
        if (this.isClosed()) {
            throw new SocketException("Socket closed");
        }
        if (obj == StandardSocketOptions.SO_SNDBUF) {
            return this.getOption(4097);
        }
        if (obj == StandardSocketOptions.SO_RCVBUF) {
            return this.getOption(4098);
        }
        if (obj == StandardSocketOptions.SO_REUSEADDR) {
            return this.getOption(4);
        }
        if (obj == StandardSocketOptions.IP_TOS) {
            return this.getOption(3);
        }
        if (obj == StandardSocketOptions.IP_MULTICAST_IF) {
            return this.getOption(31);
        }
        if (obj == StandardSocketOptions.IP_MULTICAST_TTL) {
            return Integer.valueOf(this.getTimeToLive());
        }
        if (obj == StandardSocketOptions.IP_MULTICAST_LOOP) {
            return this.getOption(18);
        }
        if (AbstractPlainDatagramSocketImpl.extendedOptions.isOptionSupported(obj)) {
            return AbstractPlainDatagramSocketImpl.extendedOptions.getOption(this.fd, obj);
        }
        throw new AssertionError("unknown option: " + obj);
    }
    
    protected abstract void datagramSocketCreate() throws SocketException;
    
    protected abstract void datagramSocketClose();
    
    protected abstract void socketSetOption(int p0, Object p1) throws SocketException;
    
    protected abstract Object socketGetOption(int p0) throws SocketException;
    
    protected abstract void connect0(InetAddress p0, int p1) throws SocketException;
    
    protected abstract void disconnect0(int p0);
    
    protected boolean nativeConnectDisabled() {
        return AbstractPlainDatagramSocketImpl.connectDisabled;
    }
    
    @Override
    abstract int dataAvailable();
    
    static {
        os = (String)AccessController.doPrivileged(new GetPropertyAction("os.name"));
        connectDisabled = AbstractPlainDatagramSocketImpl.os.contains("OS X");
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.loadLibrary("net");
                return null;
            }
        });
        extendedOptions = ExtendedSocketOptions.getInstance();
        datagramSocketOptions = datagramSocketOptions();
        multicastSocketOptions = multicastSocketOptions();
    }
}
